package cn.wise.sc.acquisition.business.service.impl;

import cn.wise.im.common.http.R;
import cn.wise.im.common.http.ResponseEnum;
import cn.wise.sc.acquisition.business.constant.Rcode;
import cn.wise.sc.acquisition.business.entity.TEquipmentCheckContent;
import cn.wise.sc.acquisition.business.entity.TEquipmentCheckDetail;
import cn.wise.sc.acquisition.business.entity.TEquipmentCheckMain;
import cn.wise.sc.acquisition.business.mapper.TEquipmentCheckContentMapper;
import cn.wise.sc.acquisition.business.mapper.TEquipmentCheckDetailMapper;
import cn.wise.sc.acquisition.business.mapper.TEquipmentCheckMapper;
import cn.wise.sc.acquisition.business.model.PageQuery;
import cn.wise.sc.acquisition.business.model.query.TEquipmentCheckDetailQuery;
import cn.wise.sc.acquisition.business.model.query.TEquipmentCheckMainQuery;
import cn.wise.sc.acquisition.business.model.vo.TEquipmentCheckMainVo;
import cn.wise.sc.acquisition.business.service.ITEquipmentCheckService;
import cn.wise.sc.acquisition.business.service.ITSysDictService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ztw
 * @since 2021-04-25
 */
@Service
@Transactional
public class TEquipmentCheckServiceImpl extends ServiceImpl<TEquipmentCheckMapper, TEquipmentCheckMain> implements ITEquipmentCheckService {

	@Resource
	private TEquipmentCheckMapper equipmentCheckMapper;
	@Resource
	private TEquipmentCheckContentMapper equipmentCheckContentMapper;
	@Resource
	private TEquipmentCheckDetailMapper equipmentCheckDetailMapper;
	@Autowired
	private ITSysDictService itSysDictService;

	/**
	 * 新增修改-设备点检主表&设备点检详情表
	 * @param query
	 * @return
	 */
	@Override
	public R checking(TEquipmentCheckMainQuery query) {
		Rcode.NOT_PARAM.assertNotNull(query);
		Rcode.NOT_PARAM.assertNotNull(query.getEquipmentID());
		Rcode.NOT_PARAM.assertNotNull(query.getEquipmentName());
		TEquipmentCheckMain checkMain = new TEquipmentCheckMain();
		BeanUtils.copyProperties(query, checkMain);

		com.baomidou.mybatisplus.extension.api.R<String> r = itSysDictService.getTagDate(null);
		if (r.getCode()!=0){
			return R.fail(ResponseEnum.FAIL,r.getMsg());
		}
		checkMain.setTag(r.getData());
		//Uid=null新增，Uid！=null 修改
		if(StringUtils.isNoneBlank(checkMain.getUid())){
			equipmentCheckMapper.updateById(checkMain);
		}else{
			equipmentCheckMapper.insert(checkMain);
		}

		//根据设备id查找点检内容表信息，插入点检详情表
		QueryWrapper<TEquipmentCheckContent> contentWrapper = new QueryWrapper<>();
		contentWrapper.eq("EquipmentID", checkMain.getEquipmentID());
		List<TEquipmentCheckContent> checkContentList = equipmentCheckContentMapper.selectList(contentWrapper);
		if(checkContentList != null && checkContentList.size()>0){
			for(TEquipmentCheckContent content : checkContentList){
				TEquipmentCheckDetail checkDetail = new TEquipmentCheckDetail();
				checkDetail.setMainUid(checkMain.getUid())
						.setEquipmentID(checkMain.getEquipmentID())
						.setEquipmentName(checkMain.getEquipmentName())
						.setDjbw(content.getDjbw())
						.setDjnr(content.getDjnr())
						.setDjbz(content.getDjbz())
						.setDjzq(content.getDjzq())
						.setTag(checkMain.getTag());
				equipmentCheckDetailMapper.insert(checkDetail);
			}
		}

		return R.ok();
	}

	/**
	 * 分页-设备点检主表
	 * @param pageQuery
	 * @param EquipmentName
	 * @return
	 */
	@Override
	public R<IPage<TEquipmentCheckMainVo>> getPage(PageQuery pageQuery, String EquipmentName, String selectMonth) {
		Map<String, Object> params = new HashMap<>();
		params.put("EquipmentName", EquipmentName);
		params.put("selectMonth", selectMonth);
		Page<TEquipmentCheckMainVo> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		IPage<TEquipmentCheckMainVo> pages = equipmentCheckMapper.getPage(page, params);
		return R.ok(pages);
	}

	/**
	 * 详情-设备点检主表
	 * @param mainUid
	 * @return
	 */
	@Override
	public R<TEquipmentCheckMain> getDetail(String mainUid) {
		Rcode.NOT_PARAM.assertNotNull(mainUid);
		TEquipmentCheckMain checkMain = equipmentCheckMapper.selectById(mainUid);
		Rcode.NOT_FOUND.assertNotNull(checkMain);
		return R.ok(checkMain);
	}

	/**
	 * 列表-设备点检详情表
	 * @param mainUid
	 * @return
	 */
	public R<List<TEquipmentCheckDetail>> getDetailList(String mainUid) {
		Rcode.NOT_PARAM.assertNotNull(mainUid);
		QueryWrapper<TEquipmentCheckDetail> detailWrapper = new QueryWrapper<>();
		detailWrapper.eq("MainUid", mainUid);
		List<TEquipmentCheckDetail> checkList = equipmentCheckDetailMapper.selectList(detailWrapper);
		return R.ok(checkList);
	}

	/**
	 * 详情-设备点检详情表
	 * @param detailUid
	 * @return
	 */
	public R<TEquipmentCheckDetail> getCheckDetail(String detailUid) {
		Rcode.NOT_PARAM.assertNotNull(detailUid);
		TEquipmentCheckDetail checkDetail = equipmentCheckDetailMapper.selectById(detailUid);
		Rcode.NOT_FOUND.assertNotNull(checkDetail);
		return R.ok(checkDetail);
	}

	/**
	 * 修改-设备点检详情表
	 * @param query
	 * @return
	 */
	public R updateDetail(TEquipmentCheckDetailQuery query)  {
		Rcode.NOT_PARAM.assertNotNull(query);
		Rcode.NOT_PARAM.assertNotNull(query.getUid());
		Rcode.NOT_PARAM.assertNotNull(query.getDjjg());
		Rcode.NOT_PARAM.assertNotNull(query.getDjrq());

		TEquipmentCheckDetail checkDetail = equipmentCheckDetailMapper.selectById(query.getUid());
		checkDetail.setDjjg(query.getDjjg())
				.setYcjl(query.getYcjl())
				.setDjr(query.getDjr())
				.setDjrq(query.getDjrq());
		equipmentCheckDetailMapper.updateById(checkDetail);
		return R.ok();
	}

	/**
	 * 删除点检信息
	 * @param mainUid
	 * @return
	 */
	public R delete(String mainUid)  {
		Rcode.NOT_PARAM.assertNotNull(mainUid);
		TEquipmentCheckMain checkMain = equipmentCheckMapper.selectById(mainUid);
		Rcode.NOT_FOUND.assertNotNull(checkMain);

		QueryWrapper<TEquipmentCheckDetail> detailWrapper = new QueryWrapper<>();
		detailWrapper.eq("MainUid", mainUid);
		equipmentCheckDetailMapper.delete(detailWrapper);
		equipmentCheckMapper.deleteById(mainUid);
		return R.ok();
	}


}
