package cn.wise.sc.acquisition.business.service.impl;

import cn.hutool.core.date.DateTime;
import cn.wise.sc.acquisition.business.constant.Rcode;
import cn.wise.sc.acquisition.business.entity.TProcessesAuxiliary;
import cn.wise.sc.acquisition.business.entity.TProcessesDigging;
import cn.wise.sc.acquisition.business.enumation.ProjectEnum;
import cn.wise.sc.acquisition.business.mapper.TProcessesAuxiliaryMapper;
import cn.wise.sc.acquisition.business.model.query.TProcessesAuxiliaryQuery;
import cn.wise.sc.acquisition.business.service.ITProcessesAuxiliaryService;
import cn.wise.sc.acquisition.business.service.ITSysDictService;
import cn.wise.sc.acquisition.business.util.BeanUtilsNewCopy;
import cn.wise.sc.acquisition.business.wrapper.page.Query;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author renchao
 * @since 2021-04-30
 */
@Service
@Slf4j
public class TProcessesAuxiliaryServiceImpl extends ServiceImpl<TProcessesAuxiliaryMapper, TProcessesAuxiliary> implements ITProcessesAuxiliaryService {

    @Autowired
    private ITSysDictService itSysDictService;
    @Resource
    private TProcessesAuxiliaryMapper processesAuxiliaryMapper;

    @Override
    public R getPage(TProcessesAuxiliaryQuery tProcessesAuxiliaryQuery, Query query) {
        log.info(JSON.toJSONString(tProcessesAuxiliaryQuery));
        //参数校验
        //分页
        Page<TProcessesAuxiliary> page = new Page<>(query.getPageNum(), query.getPageSize());
        //条件封装
        QueryWrapper<TProcessesAuxiliary> queryWrapper = new QueryWrapper<>();
        queryWrapper.select(
                ProjectEnum.TProcessesAuxiliary.UID.getLabel(),
                ProjectEnum.TProcessesAuxiliary.EQUIPMENT_ID.getLabel(),
                ProjectEnum.TProcessesAuxiliary.EQUIPMENT_NAME.getLabel(),
                ProjectEnum.TProcessesAuxiliary.CONSUMEDATE.getLabel(),
                ProjectEnum.TProcessesAuxiliary.NAME.getLabel(),
                ProjectEnum.TProcessesAuxiliary.AMOUNT.getLabel(),
                ProjectEnum.TProcessesAuxiliary.TOTAL.getLabel(),
                ProjectEnum.TProcessesAuxiliary.CL.getLabel(),
                ProjectEnum.TProcessesAuxiliary.DH.getLabel(),
                ProjectEnum.TProcessesAuxiliary.CLLYR.getLabel(),
                ProjectEnum.TProcessesAuxiliary.FLAG.getLabel(),
                ProjectEnum.TProcessesAuxiliary.TAG.getLabel(),
                ProjectEnum.TProcessesAuxiliary.BZ.getLabel()
        );
        queryWrapper.eq(StringUtils.isNotBlank(tProcessesAuxiliaryQuery.getEquipmentID()),
                ProjectEnum.TProcessesAuxiliary.EQUIPMENT_ID.getLabel(), tProcessesAuxiliaryQuery.getEquipmentID());
        //查询
        Page<TProcessesAuxiliary> tSampleListPage = baseMapper.selectPage(page, queryWrapper);
        return R.ok(tSampleListPage);
    }

    @Override
    public R updateByUid(TProcessesAuxiliaryQuery query) {

        log.info(JSON.toJSONString(query));
        //参数校验
        Rcode.NOT_PARAM.assertNotNull(query);
        Rcode.NOT_PARAM.assertNotEmpty(query.getUid());
        //判断数据是否存在
        TProcessesAuxiliary temp = baseMapper.selectById(query.getUid());
        if (temp == null) {
            return R.failed("辅助工序中->数据不存在，uid：" + query.getUid());
        }
        //tag不能修改
        if (StringUtils.isNotBlank(query.getTag())) {
            return R.failed("辅助工序中->tag为自动生成字段，不能修改");
        }
        //当月累计油量不为空时，需要判断。当月累计油量不为0时，当月累计产量不能为0 因为dh 由当月累计油量/当月累计产量 计算而来，分母不能为0
        if ((query.getAmount() != null && query.getAmount() != 0)
                && query.getCl() != null
                && query.getCl() == 0) {
            return R.failed("辅助工序中->当要同时修改当月累计油量和当月累计油量时, 当月累计油量Ljyl 不为0时，当月累计产量Dyljcl不能为0,因为dh 由当月累计油量/当月累计产量 计算而来，分母不能为0");
        }
        //条件封装
        BeanUtilsNewCopy.copyPropertiesIgnoreNull(query, temp);

        QueryWrapper<TProcessesAuxiliary> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectEnum.TProcessesAuxiliary.UID.getLabel(), query.getUid());
        //计算单耗 防止失去精度，先转成字符串
        float dh = temp.getAmount() / temp.getCl();

        temp.setDh(Float.valueOf(String.valueOf(dh)));
        //修改
        int update = baseMapper.update(temp, queryWrapper);
        if (update > 0) {
            return R.ok("辅助工序中->修改成功");
        } else {
            log.error("辅助工序中->修改失败:{}", JSON.toJSONString(temp));
            return R.failed("辅助工序中->修改失败");
        }

    }

    @Override
    public R insertTProcessesAuxiliary(TProcessesAuxiliaryQuery query) {
        log.info(JSON.toJSONString(query));
        //参数校验
        Rcode.NOT_PARAM.assertNotNull(query);
        Rcode.NOT_PARAM.assertNotNull(query.getAmount());
        Rcode.NOT_PARAM.assertNotNull(query.getCl());
        //uid自动生成 不能手动设置
        if (query.getUid() != null) {
            return R.failed("辅助工序中->uid自动生成 不能手动设置");
        }
        //tag不能修改
        if (StringUtils.isNotBlank(query.getTag())) {
            return R.failed("辅助工序中->tag为自动生成字段，不能添加");
        }
        //单耗为自动计算
        if (query.getDh() != null) {
            return R.failed("辅助工序中->dh为自动计算字段，不能添加");
        }
        //当月累计油量不为0时，当月累计产量不能为0
        if (query.getAmount() != 0 && query.getCl() == 0) {
            return R.failed("辅助工序中->当月累计油量Ljyl 不为0时，当月累计产量Dyljcl不能为0,分母不能为0");
        }
        query.setFlag(false);
        //条件封装
        TProcessesAuxiliary tProcessesAuxiliary = new TProcessesAuxiliary();
        BeanUtils.copyProperties(query, tProcessesAuxiliary);
        //设置tag
        R<String> r1 = itSysDictService.getTagDate(null);
        if (r1.getCode() != 0) {
            return R.failed(r1.getMsg());
        }
        tProcessesAuxiliary.setTag(r1.getData());
        //设置单耗
        if (query.getAmount() == 0) {
            tProcessesAuxiliary.setDh(0f);
        } else {
            //强转失去精度，借助字符串
            float dh = query.getAmount() / query.getCl();
            tProcessesAuxiliary.setDh(Float.valueOf(String.valueOf(dh)));
        }
        //增加
        log.info(JSON.toJSONString(tProcessesAuxiliary));
        int insert = baseMapper.insert(tProcessesAuxiliary);
        if (insert > 0) {
            return R.ok("辅助工序中->增加成功");
        } else {
            log.error("辅助工序中->增加失败: {}", JSON.toJSONString(tProcessesAuxiliary));
            return R.failed("辅助工序中->增加失败");
        }
    }


    /**
     * 查询最新的记录
     * @return
     */
    @Override
    public cn.wise.im.common.http.R<TProcessesAuxiliary> getLatest(){
        QueryWrapper<TProcessesAuxiliary> qw = new QueryWrapper<>();
        qw.orderByDesc("ConsumeDate");
        List<TProcessesAuxiliary> list = processesAuxiliaryMapper.selectList(qw);
        if(list != null && list.size()>0){
            TProcessesAuxiliary processesAuxiliary = list.get(0);
            return cn.wise.im.common.http.R.ok(processesAuxiliary);
        }else{
            return cn.wise.im.common.http.R.ok(null);
        }
    }







}
