package cn.wise.sc.acquisition.business.service.impl;

import cn.wise.im.common.http.R;
import cn.wise.sc.acquisition.business.constant.Rcode;
import cn.wise.sc.acquisition.business.entity.*;
import cn.wise.sc.acquisition.business.enumation.FileExt;
import cn.wise.sc.acquisition.business.mapper.TSafeTroubleImgMapper;
import cn.wise.sc.acquisition.business.mapper.TSafeTroubleMapper;
import cn.wise.sc.acquisition.business.model.LoginUser;
import cn.wise.sc.acquisition.business.model.PageQuery;
import cn.wise.sc.acquisition.business.model.query.TMaterialOutQuery;
import cn.wise.sc.acquisition.business.model.query.TSafeTroubleQuery;
import cn.wise.sc.acquisition.business.model.vo.TSafeTroubleVo;
import cn.wise.sc.acquisition.business.service.ITSafeTroubleService;
import cn.wise.sc.acquisition.business.util.ImageUtil;
import cn.wise.sc.acquisition.business.util.dfs.FastDFSUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.swagger.annotations.ApiOperation;
import io.swagger.models.auth.In;
import org.apache.tomcat.util.http.fileupload.servlet.ServletFileUpload;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author renchao
 * @since 2021-04-27
 */
@Service
@Transactional
public class TSafeTroubleServiceImpl extends ServiceImpl<TSafeTroubleMapper, TSafeTrouble> implements ITSafeTroubleService {

	@Resource
	private TSafeTroubleMapper safeTroubleMapper;
	@Resource
	private TSafeTroubleImgMapper safeTroubleImgMapper;

	/**
	 * 安全台账分页
	 * @param pageQuery
	 * @return
	 */
	@Override
	public R<IPage<TSafeTrouble>> getPage(PageQuery pageQuery) {
		QueryWrapper<TSafeTrouble> qw = new QueryWrapper<>();
		qw.orderByDesc("Fxrq");
		IPage<TSafeTrouble> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		page = safeTroubleMapper.selectPage(page, qw);

	/*	QueryWrapper<TSafeTroubleImg> imgWrapper = new QueryWrapper<>();
		List<TSafeTroubleImg> checkList = safeTroubleImgMapper.selectList(imgWrapper);

		List<TSafeTroubleVo> voList = new ArrayList<>();
		List<TSafeTrouble> list = page.getRecords();
		for(TSafeTrouble safeTrouble : list){
			TSafeTroubleVo safeTroubleVo = new TSafeTroubleVo();
			BeanUtils.copyProperties(safeTrouble, safeTroubleVo);
			//整改前照片
			List<TSafeTroubleImg> oldImgs =  checkList.stream()
					.filter(arg -> arg.getTroubleUid().equals(safeTrouble.getUid()) && arg.getImgType().equals(TSafeTroubleImg.ImgType.YH)).collect(Collectors.toList());
			safeTroubleVo.setOldImgs(oldImgs);
			//整改后照片
			List<TSafeTroubleImg> newImgs =  checkList.stream()
					.filter(arg -> arg.getTroubleUid().equals(safeTrouble.getUid()) && arg.getImgType().equals(TSafeTroubleImg.ImgType.ZG)).collect(Collectors.toList());
			safeTroubleVo.setNewImgs(newImgs);
			voList.add(safeTroubleVo);
		}
		IPage<TSafeTroubleVo> pagevo = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
		pagevo.setRecords(voList);*/

		return R.ok(page);
	}


	/**
	 * 添加隐患
	 * @param query
	 * @return
	 */
	@Override
	@Transactional
	public R save(TSafeTroubleQuery query) {
		Rcode.NOT_PARAM.assertNotNull(query);
		TSafeTrouble safeTrouble = new TSafeTrouble();
		BeanUtils.copyProperties(query, safeTrouble);
		safeTroubleMapper.insert(safeTrouble);

		if(query.getBList()!=null && query.getBList().size()>0){
			for (byte[] bytes : query.getBList()) {
				TSafeTroubleImg safeTroubleImg = new TSafeTroubleImg();
				safeTroubleImg.setTroubleUid(safeTrouble.getUid())
						.setImg(bytes)
						.setImgType(TSafeTroubleImg.ImgType.YH);
				safeTroubleImgMapper.insert(safeTroubleImg);
			}
		}
		return R.ok();
	}



	/**
	 * 隐患整改
	 * @param query
	 * @return
	 */
	@Override
	public R update(TSafeTroubleQuery query) {
		Rcode.NOT_PARAM.assertNotNull(query);
		Rcode.NOT_PARAM.assertNotNull(query.getUid());
		TSafeTrouble safeTrouble = safeTroubleMapper.selectById(query.getUid());
		Rcode.NOT_FOUND.assertNotNull(safeTrouble);
		BeanUtils.copyProperties(query, safeTrouble);
		safeTroubleMapper.updateById(safeTrouble);
		if(query.getBList()!=null && query.getBList().size()>0){
			//删除之前的所有图片
			QueryWrapper<TSafeTroubleImg> stiWrapper = new QueryWrapper<>();
			stiWrapper.eq("TroubleUid", safeTrouble.getUid());
			stiWrapper.eq("ImgType", TSafeTroubleImg.ImgType.YH);
			safeTroubleImgMapper.delete(stiWrapper);

			//保存没有删除的 和新添加的图片
			for (byte[] bytes : query.getBList()) {
				TSafeTroubleImg safeTroubleImg = new TSafeTroubleImg();
				safeTroubleImg.setTroubleUid(safeTrouble.getUid())
						.setImg(bytes)
						.setImgType(TSafeTroubleImg.ImgType.YH);
				safeTroubleImgMapper.insert(safeTroubleImg);
			}
		}

		if(query.getZgbList()!=null && query.getZgbList().size()>0){
			//删除之前的所有图片
			QueryWrapper<TSafeTroubleImg> stiWrapper = new QueryWrapper<>();
			stiWrapper.eq("TroubleUid", safeTrouble.getUid());
			stiWrapper.eq("ImgType", TSafeTroubleImg.ImgType.ZG);
			safeTroubleImgMapper.delete(stiWrapper);

			//保存没有删除的 和新添加的图片
			for (byte[] bytes : query.getZgbList()) {
				TSafeTroubleImg safeTroubleImg = new TSafeTroubleImg();
				safeTroubleImg.setTroubleUid(safeTrouble.getUid())
						.setImg(bytes)
						.setImgType(TSafeTroubleImg.ImgType.ZG);
				safeTroubleImgMapper.insert(safeTroubleImg);
			}
		}

		return R.ok();
	}


	/**
	 * 隐患详情
	 * @param Uid
	 * @return
	 */
	@Override
	public R<TSafeTroubleVo> getDetail(String Uid) {
		Rcode.NOT_PARAM.assertNotNull(Uid);

		TSafeTrouble safeTrouble = safeTroubleMapper.selectById(Uid);
		Rcode.NOT_FOUND.assertNotNull(safeTrouble);
		TSafeTroubleVo safeTroubleVo = new TSafeTroubleVo();
		BeanUtils.copyProperties(safeTrouble, safeTroubleVo);

		QueryWrapper<TSafeTroubleImg> imgWrapper = new QueryWrapper<>();
		imgWrapper.eq("TroubleUid", safeTroubleVo.getUid());
		List<TSafeTroubleImg> checkList = safeTroubleImgMapper.selectList(imgWrapper);
		//整改前照片
		List<TSafeTroubleImg> oldImgs =  checkList.stream()
				.filter(arg -> arg.getImgType().equals(TSafeTroubleImg.ImgType.YH)).collect(Collectors.toList());
		safeTroubleVo.setOldImgs(oldImgs);
		//整改后照片
		List<TSafeTroubleImg> newImgs =  checkList.stream()
				.filter(arg -> arg.getImgType().equals(TSafeTroubleImg.ImgType.ZG)).collect(Collectors.toList());
		safeTroubleVo.setNewImgs(newImgs);
		return R.ok(safeTroubleVo);
	}


}
